"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pngjs_1 = require("pngjs");
const rect_1 = require("./rect");
const snappit_1 = require("./snappit");
const SVR_ID = "added-by-snappit-visual-regression";
/**
 * Class for an Element Screenshot.
 */
class ElementScreenshot {
    /**
     * Creates a new `ElementScreenshot` object.
     * @param element `WebElement` to screenshot, or whose contents to screenshot (see below).
     * @param elementContent `boolean` indicating whether or not screenshot the element's contents instead,
     *   only valid for scrolling elements.
     */
    constructor(element, elementContent = false) {
        this.element = element;
        this.elementContent = elementContent;
        /**
         * Drops scrollbars in all modern browsers.
         */
        this.dropScrollbars = () => this.driver.executeScript(() => {
            const head = document.querySelector("head");
            const style = document.createElement("style");
            style.id = arguments[0];
            style.type = "text/css";
            style.innerText = `
                    *::-webkit-scrollbar { display: none !important; }
                    :not(body){overflow: -moz-scrollbars-none !important;}
                `;
            head.appendChild(style);
        }, SVR_ID).catch((err) => {
            err.message = "Error attempting to drop scrollbars before screenshot: " + err.message;
            throw err;
        });
        /**
         * Retrieve the element's viewport element.  If there is no parent return the doc element.
         * This attempts to handle fixed or absolute positioned elements by falling back to the
         * doc element as parent.
         */
        this.getViewportElement = () => __awaiter(this, void 0, void 0, function* () {
            return this.driver.executeScript(() => {
                const isScrollable = (elem) => elem.scrollHeight !== elem.clientHeight ||
                    elem.scrollWidth !== elem.clientWidth;
                const isAbsolute = (elem) => ["absolute", "fixed"].indexOf(window.getComputedStyle(elem).position) !== -1;
                const docElement = document.documentElement || document.getElementsByTagName("html")[0];
                const element = arguments[0];
                let parent = element.parentElement;
                if (!parent || isAbsolute(element)) {
                    return docElement;
                }
                while (parent !== docElement && !isScrollable(parent)) {
                    if (isAbsolute(parent)) {
                        return docElement;
                    }
                    parent = parent.parentElement;
                }
                /**
                 * If the parent is the body element, we need to dodge it.
                 */
                const bodyElement = document.getElementsByTagName("body")[0];
                return parent === bodyElement ? docElement : parent;
            }, this.element);
        });
        /**
         * Undo the drop scrollbars style.
         */
        this.removeDropScrollbars = () => this.driver.executeScript(() => document.getElementById(arguments[0]).remove(), SVR_ID).catch((err) => {
            err.message = "Error replacing scrollbars after screenshot: " + err.message;
            throw err;
        });
        /**
         * Scroll to the specified location on screen accounting for devicePixelRatio.  This method assumes viewport
         * coordinates, so you will need to adjust your inputs appropriately.
         */
        this.scrollToLocation = (viewport, left, top) => this.driver.executeScript(() => {
            const pixelRatio = window.devicePixelRatio;
            const x = arguments[1] / pixelRatio;
            const y = arguments[2] / pixelRatio;
            const element = arguments[0];
            element.scrollTo(x, y);
        }, viewport, left, top);
        this.driver = element.getDriver();
    }
    /**
     * Takes the element screenshot.
     *
     * If `this.elementContent` is set to true, the element will be used as a viewport and the elements contents
     * will be screenshotted instead.
     */
    take() {
        return __awaiter(this, void 0, void 0, function* () {
            // Drop scrollbars
            yield this.dropScrollbars();
            // Get viewport
            const viewport = this.elementContent
                ? this.element
                : yield this.getViewportElement();
            const viewportRect = yield new rect_1.Rect().forViewportElement(viewport);
            /* Position the element in the viewport by scrolling to the top left of the viewport, then attempting to
             * scroll the viewport to the relative top left of the element.  Since scrolling is based on the viewport
             * element, we'll adjust the initial position to be relative to the viewport.
             */
            yield this.scrollToLocation(viewport, 0, 0);
            const initialRect = this.elementContent
                ? (yield new rect_1.Rect().forElementContent(this.element)).relativeTo(viewportRect)
                : (yield new rect_1.Rect().forElement(this.element)).relativeTo(viewportRect);
            yield this.scrollToLocation(viewport, initialRect.left, initialRect.top);
            // Create base screenshot.
            const elementScreenshot = new pngjs_1.PNG(initialRect.duplicate());
            const cursor = {
                x: 0,
                y: 0,
            };
            // Until we've filled up the entire elementScreenshot
            while (cursor.y < initialRect.height) {
                while (cursor.x < initialRect.width) {
                    // Take a new screenshot at the current location
                    const screenshot = yield this.screenshot();
                    const rect = this.elementContent
                        ? (yield new rect_1.Rect().forElementContent(this.element))
                        : yield new rect_1.Rect().forElement(this.element);
                    /**
                     * Given the offset information in rect, determine the intersection with the current viewport's
                     * position on the screen.
                     *
                     * The formula for the intersection of two rectangles is the max of the left and top offsets,
                     * and the min of the right and bottom offsets.
                     *
                     * To find the width and the height, we have to subtract top from bottom, left from right.
                     */
                    const sourceX = Math.max(rect.left, viewportRect.left);
                    const sourceY = Math.max(rect.top, viewportRect.top);
                    const width = Math.min(rect.right, viewportRect.right) - sourceX;
                    const height = Math.min(rect.bottom, viewportRect.bottom) - sourceY;
                    /**
                     * If we would overwrite width/height by too much, adjust the cursor.  This is slightly less
                     * efficient but conceptually simpler than adjusting the source and width.
                     */
                    if (width + cursor.x > initialRect.width) {
                        cursor.x = initialRect.width - width;
                    }
                    if (height + cursor.y > initialRect.height) {
                        cursor.y = initialRect.height - height;
                    }
                    pngjs_1.PNG.bitblt(screenshot, elementScreenshot, sourceX, sourceY, width, height, cursor.x, cursor.y);
                    cursor.x += width;
                    if (cursor.x >= initialRect.width) {
                        cursor.x = 0;
                        cursor.y += height;
                        break;
                    }
                    this.scrollToLocation(viewport, initialRect.left + cursor.x, initialRect.top + cursor.y);
                }
                this.scrollToLocation(viewport, initialRect.left + cursor.x, initialRect.top + cursor.y);
            }
            // Restore scrollbars
            yield this.removeDropScrollbars();
            return elementScreenshot;
        });
    }
    screenshot() {
        return __awaiter(this, void 0, void 0, function* () {
            return pngjs_1.PNG.sync.read(new Buffer(yield this.driver.takeScreenshot(), "base64"));
        });
    }
}
function default_1(element, elementContent) {
    return __awaiter(this, void 0, void 0, function* () {
        /**
         * We want to use the documentElement, a.k.a. html tag, instead of the body tag if that is passed.  The
         * body tag does not properly handle scrolling, and is essentially the same for the purposes of computing
         * the area to screenshot.
         */
        if ((yield element.getTagName()).toLowerCase() === "body") {
            element = snappit_1.$("html");
        }
        return new ElementScreenshot(element, elementContent).take();
    });
}
exports.default = default_1;
//# sourceMappingURL=elementScreenshot.js.map