"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// Provides an array whose member types are deterministic at compile time.
class StringArray extends Array {
}
const RECT_KEYS = new StringArray("bottom", "height", "left", "right", "top", "width");
class Rect {
    constructor() {
        this.bottom = 0;
        this.height = 0;
        this.left = 0;
        this.right = 0;
        this.top = 0;
        this.width = 0;
        /**
         * Wraps getBoundingClientRect.
         * @param element `WebElement` to retrieve the rect for.
         */
        this.getBoundingRect = (element) => __awaiter(this, void 0, void 0, function* () {
            return element.getDriver().executeScript(() => {
                const rect = arguments[0].getBoundingClientRect();
                const ratio = window.devicePixelRatio;
                return {
                    bottom: ratio * rect.bottom,
                    height: ratio * rect.height,
                    left: ratio * rect.left,
                    right: ratio * rect.right,
                    top: ratio * rect.top,
                    width: ratio * rect.width,
                };
            }, element);
        });
        /**
         * Returns the client dimensions and offsets.
         * @param element `WebElement` to retrieve the rect for.
         */
        this.getClientRect = (element) => __awaiter(this, void 0, void 0, function* () {
            return element.getDriver().executeScript(() => {
                const elem = arguments[0];
                const ratio = window.devicePixelRatio;
                return {
                    height: ratio * elem.clientHeight,
                    left: ratio * elem.clientLeft,
                    top: ratio * elem.clientTop,
                    width: ratio * elem.clientWidth,
                };
            }, element);
        });
        /**
         * Returns the scroll offsets.
         * @param element `WebElement` to retrieve the rect for.
         */
        this.getScrollRect = (element) => __awaiter(this, void 0, void 0, function* () {
            return element.getDriver().executeScript(() => {
                const elem = arguments[0];
                const ratio = window.devicePixelRatio;
                return {
                    height: ratio * elem.scrollHeight,
                    width: ratio * elem.scrollWidth,
                };
            }, element);
        });
        /**
         * Determines if an element is the document element.
         * @param element `WebElement` to test.
         */
        this.isDocElement = (element) => __awaiter(this, void 0, void 0, function* () {
            return element.getDriver().executeScript(() => {
                const docElement = document.documentElement || document.getElementsByTagName("html")[0];
                return arguments[0] === docElement;
            }, element);
        });
    }
    /**
     * Duplicate the Rect to a new object.
     */
    duplicate() {
        const newRect = new Rect();
        for (const key of RECT_KEYS) {
            newRect[key] = this[key];
        }
        return newRect;
    }
    /**
     * Retrieve the Rect corresponding to the element's bounding box accounting for pixelRatio.
     * This method uses `getBoundingClientRect` but will round away subpixel values to the nearest pixel.
     * @param element The `WebElement` to fetch the Rect for.
     */
    forElement(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const clientRect = yield this.getBoundingRect(element);
            return this.duplicate.apply(clientRect).round();
        });
    }
    /**
     * Retrieve the Rect corresponding to the viewport element's content. This is obtained by looking at the scroll
     * width and height of the viewport element and its scroll offsets, then mapping to the rect obtained via the
     * `forViewportElement` call.
     * @param element The `WebElement` to fetch the content Rect for.
     */
    forElementContent(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const newRect = yield this.forViewportElement(element);
            const scrollRect = yield this.getScrollRect(element);
            newRect.bottom = newRect.top + scrollRect.height;
            newRect.right = newRect.left + scrollRect.width;
            newRect.height = scrollRect.height;
            newRect.width = scrollRect.width;
            return newRect.round();
        });
    }
    /**
     * Retrieve the Rect corresponding to the viewport element's bounding box accounting for pixelRatio.
     * The viewport bounding box is defined as the bounding box of the area through which the element's content is
     * visible. This is found by adjusting the element's boundingRect by it's clientRect.
     * @param element The `WebElement` to fetch the viewport Rect for.
     */
    forViewportElement(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const newRect = new Rect();
            const { left, top, height, width } = yield this.getClientRect(element);
            if (yield this.isDocElement(element)) {
                newRect.left = 0;
                newRect.top = 0;
                newRect.right = newRect.width = width;
                newRect.bottom = newRect.height = height;
            }
            else {
                const rect = yield this.getBoundingRect(element);
                newRect.height = height;
                newRect.width = width;
                newRect.top = rect.top + top;
                newRect.bottom = rect.top + top + height;
                newRect.left = rect.left + left;
                newRect.right = rect.left + left + width;
            }
            return newRect.round();
        });
    }
    /**
     * Transform position to relative pixels by the reference point (left, top).
     * @param rect `Rect` object containing the top, left coordinates.
     */
    relativeTo({ left, top }) {
        const newRect = this.duplicate();
        newRect.top -= top;
        newRect.bottom -= top;
        newRect.left -= left;
        newRect.right -= left;
        return newRect;
    }
    /**
     * Round all of the Rect keys.
     */
    round() {
        const newRect = new Rect();
        for (const key of RECT_KEYS) {
            newRect[key] = Math.round(this[key]);
        }
        return newRect;
    }
}
exports.Rect = Rect;
//# sourceMappingURL=rect.js.map