"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const _ = require("lodash");
const path = require("path");
const selenium_webdriver_1 = require("selenium-webdriver");
const config_1 = require("./config");
const errors_1 = require("./errors");
const blackout = require("./blackout");
const getDriver_1 = require("./getDriver");
const screenshot_1 = require("./screenshot");
/**
 * Snappit exposes shorthand to its public `$` and `snap` methods.  These methods
 * are only valid with a current Snappit session.  We declare their initial state to return
 * a `NoDriverSessionException` and modify them in the `start()` and `stop()` routines.
 */
let shorthandInstance;
function snap(name, element, opts = {
    elementContent: false,
    hide: [],
}) {
    return __awaiter(this, void 0, void 0, function* () {
        if (shorthandInstance) {
            return shorthandInstance.snap(name, element, opts);
        }
        throw new errors_1.NoDriverSessionException();
    });
}
exports.snap = snap;
/* tslint:disable-next-line:no-namespace */
(function (snap) {
    function configure(config) {
        if (shorthandInstance) {
            return shorthandInstance.configureSnap(config);
        }
        throw new errors_1.NoDriverSessionException();
    }
    snap.configure = configure;
})(snap = exports.snap || (exports.snap = {}));
function $(selector) {
    if (shorthandInstance) {
        return shorthandInstance.$(selector);
    }
    throw new errors_1.NoDriverSessionException();
}
exports.$ = $;
class Snappit {
    constructor(config, driver) {
        if (driver instanceof selenium_webdriver_1.WebDriver) {
            config.useProvidedDriver = true;
        }
        this.config = config_1.prepareConfig(config);
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.driver) {
                this.driver = yield getDriver_1.getDriver(this.config);
            }
            // Update the exported shorthand methods
            shorthandInstance = this;
            return this.driver;
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            // Update the exported shorthand methods
            shorthandInstance = undefined;
            try {
                yield this.driver.close();
                yield this.driver.quit();
            }
            catch (e) {
                // Ignore the driver quit error
            }
            this.driver = undefined;
        });
    }
    $(selector) {
        return this.driver.findElement(selenium_webdriver_1.By.css(selector));
    }
    handleException(error) {
        if (this.config.logException.indexOf(error.id) >= 0) {
            /* tslint:disable-next-line:no-console */
            console.log(error.message);
        }
        else {
            throw error;
        }
    }
    snap(name, element, { elementContent = false, hide = [], } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const filePath = yield screenshot_1.Screenshot.buildPath(name, this.driver, this.config.screenshotsDir);
            const shortPath = path.relative(process.cwd(), filePath);
            if (hide.length) {
                yield blackout.hideElements(this.driver, hide);
            }
            const newShot = yield screenshot_1.Screenshot.take(this.driver, element, elementContent);
            if (hide.length) {
                yield blackout.unhideElements(this.driver, hide);
            }
            // Baseline image exists
            if (fs.existsSync(filePath)) {
                const oldShot = screenshot_1.Screenshot.fromPath(filePath);
                const diff = newShot.percentDiff(oldShot);
                if (!newShot.isSameSize(oldShot)) {
                    newShot.saveToPath(filePath);
                    this.handleException(new errors_1.ScreenshotSizeDifferenceException(shortPath));
                }
                else if (diff > this.config.threshold) {
                    const prettyDiff = `${(diff * 100).toFixed(2)}% / ${(this.config.threshold * 100).toFixed(2)}%`;
                    const message = `${shortPath} (${prettyDiff})`;
                    newShot.saveToPath(filePath);
                    this.handleException(new errors_1.ScreenshotMismatchException(message));
                }
                else {
                    // Do not save the screenshot if no changes detected, simply update timestamps
                    const now = new Date();
                    fs.utimesSync(filePath, now, now);
                }
                // No baseline image
            }
            else {
                newShot.saveToPath(filePath);
                this.handleException(new errors_1.ScreenshotNoBaselineException(shortPath));
            }
        });
    }
    configureSnap(config) {
        this.config = config_1.prepareConfig(_.defaults(_.cloneDeep(config), this.config));
    }
    setSauceLabsJobResult(passed) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.driver.executeScript(`sauce:job-result="${passed}"`);
        });
    }
    setSauceLabsAnnotation(annotation) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.driver.executeScript(`sauce:context="${annotation}"`);
        });
    }
}
exports.Snappit = Snappit;
//# sourceMappingURL=snappit.js.map