"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const fs = require("fs-extra");
const pngjs_1 = require("pngjs");
const selenium_webdriver_1 = require("selenium-webdriver");
const errors_1 = require("../src/errors");
const snappit_1 = require("../src/snappit");
function browserTest(suiteName, config, suiteFn = describe) {
    suiteFn(suiteName, function () {
        let driver;
        let snappit;
        function compareImageDimensions(baseline, current) {
            return __awaiter(this, void 0, void 0, function* () {
                const ratio = (yield driver.executeScript(() => window.devicePixelRatio));
                const originalPng = pngjs_1.PNG.sync.read(fs.readFileSync(baseline));
                const savedPng = pngjs_1.PNG.sync.read(fs.readFileSync(current));
                chai_1.expect(originalPng.width * ratio).to.eql(savedPng.width);
                chai_1.expect(originalPng.height * ratio).to.eql(savedPng.height);
            });
        }
        this.timeout(15000);
        this.slow(2500);
        before(() => {
            config.screenshotsDir = `./test/screenshots/${suiteName}`;
            config.threshold = 0.1;
            snappit = new snappit_1.Snappit(config);
        });
        describe("before 'snappit.start()'", () => {
            it("should throw an error on invoking '$'", () => {
                const fn = snappit_1.$;
                chai_1.expect(fn).to.throw(errors_1.NoDriverSessionException);
            });
            it("should throw an error on invoking 'snap'", () => __awaiter(this, void 0, void 0, function* () {
                const error = yield snappit_1.snap("foo").catch((err) => err);
                chai_1.expect(error).to.be.an.instanceOf(errors_1.NoDriverSessionException);
            }));
        });
        describe("driver initialization", () => {
            it("should create a driver instance", () => __awaiter(this, void 0, void 0, function* () {
                // Snappit will throw if there is a problem in driver creation.
                driver = yield snappit.start();
                yield driver;
            }));
            it("should navigate to the localhost page", () => __awaiter(this, void 0, void 0, function* () {
                driver.get("http://localhost:8080/");
                chai_1.expect(yield snappit_1.$("#color-div").isDisplayed()).to.eql(true);
            }));
            it("should terminate the driver instance", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
        });
        describe("after 'snappit.stop()'", () => {
            before(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.start();
                yield snappit.stop();
            }));
            it("should throw an error on invoking '$'", () => {
                const fn = snappit_1.$;
                chai_1.expect(fn).to.throw(errors_1.NoDriverSessionException);
            });
            it("should throw an error on invoking 'snap'", () => __awaiter(this, void 0, void 0, function* () {
                const error = yield snappit_1.snap("foo").catch((err) => err);
                chai_1.expect(error).to.be.an.instanceOf(errors_1.NoDriverSessionException);
            }));
        });
        describe("when setting initial viewport size", () => __awaiter(this, void 0, void 0, function* () {
            const width = 1200;
            const height = 900;
            before(() => __awaiter(this, void 0, void 0, function* () {
                config.initialViewportSize = [width, height];
                snappit = new snappit_1.Snappit(config);
                driver = yield snappit.start();
                yield driver.get("http://localhost:8080/");
            }));
            after(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
            it("should set the viewport size to the desired width", () => __awaiter(this, void 0, void 0, function* () {
                chai_1.expect((yield driver.manage().window().getSize()).width).to.equal(width);
            }));
            it("should set the viewport size to the desired height", () => __awaiter(this, void 0, void 0, function* () {
                chai_1.expect((yield driver.manage().window().getSize()).height).to.equal(height);
            }));
        }));
        describe("screenshots", () => {
            const suitePath = `./test/screenshots/${suiteName.split(" ").join("-")}`;
            before(() => __awaiter(this, void 0, void 0, function* () {
                config.logException = [errors_1.ScreenshotExceptionName.NO_BASELINE];
                snappit = new snappit_1.Snappit(config);
                driver = yield snappit.start();
                yield driver.get("http://localhost:8080/");
            }));
            after(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
            it("should throw an error if the screenshot does not exist", () => __awaiter(this, void 0, void 0, function* () {
                snappit_1.snap.configure({ logException: [] });
                const error = yield snappit_1.snap("does-not-exist.png", snappit_1.$("#color-div")).catch((err) => err);
                snappit_1.snap.configure({ logException: [errors_1.ScreenshotExceptionName.NO_BASELINE] });
                chai_1.expect(error).to.be.an.instanceof(errors_1.ScreenshotNoBaselineException);
            }));
            it("should throw an error if the screenshot is a different size", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit_1.snap("different-size.png", snappit_1.$("body"));
                const error = yield snappit_1.snap("different-size.png", snappit_1.$("#color-div")).catch((err) => err);
                chai_1.expect(error).to.be.an.instanceof(errors_1.ScreenshotSizeDifferenceException);
            }));
            it("should throw an error if the screenshot is different above threshold", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit_1.snap("different-above-threshold.png", snappit_1.$("#color-div"));
                snappit_1.$("#toggle-button").click();
                const error = yield snappit_1.snap("different-above-threshold.png", snappit_1.$("#color-div")).catch((err) => err);
                chai_1.expect(error).to.be.an.instanceof(errors_1.ScreenshotMismatchException);
            }));
            it("should not throw an error if the screenshot is different below threshold", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit_1.snap("different-below-threshold.png", snappit_1.$("#color-div"));
                snappit_1.$("#border-button").click();
                yield snappit_1.snap("different-below-threshold.png", snappit_1.$("#color-div"));
            }));
            it("should not throw an error if the screenshot shows no difference", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit_1.snap("no-difference.png", snappit_1.$("#color-div"));
                yield snappit_1.snap("no-difference.png", snappit_1.$("#color-div"));
            }));
        });
        describe("scrolling elements", () => {
            const suitePath = `./test/screenshots/${suiteName.split(" ").join("-")}`;
            before(() => __awaiter(this, void 0, void 0, function* () {
                config.logException = [errors_1.ScreenshotExceptionName.NO_BASELINE];
                snappit = new snappit_1.Snappit(config);
                driver = yield snappit.start();
            }));
            after(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
            it("should take a snapshot of an element that is too wide", () => __awaiter(this, void 0, void 0, function* () {
                yield driver.get("http://localhost:8080/too-wide");
                const imageName = "too-wide.png";
                const baseline = `./test/public/img/${imageName}`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#too-wide"));
                yield compareImageDimensions(baseline, current);
            }));
            it("should take a snapshot of an element that is too tall", () => __awaiter(this, void 0, void 0, function* () {
                yield driver.get("http://localhost:8080/too-tall");
                const imageName = "too-tall.png";
                const baseline = `./test/public/img/${imageName}`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#too-tall"));
                yield compareImageDimensions(baseline, current);
            }));
            it("should take a snapshot of an element that is too wide and too tall", () => __awaiter(this, void 0, void 0, function* () {
                yield driver.get("http://localhost:8080/too-wide-too-tall");
                const imageName = "too-wide-too-tall.png";
                const baseline = `./test/public/img/${imageName}`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#too-wide-too-tall"));
                yield compareImageDimensions(baseline, current);
            }));
        });
        describe("internal scrolling elements", () => {
            const suitePath = `./test/screenshots/${suiteName.split(" ").join("-")}`;
            before(() => __awaiter(this, void 0, void 0, function* () {
                config.logException = [errors_1.ScreenshotExceptionName.NO_BASELINE];
                snappit = new snappit_1.Snappit(config);
                driver = yield snappit.start();
                yield driver.get("http://localhost:8080/internal-scroll.html");
            }));
            after(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
            it("should take a screenshot of an element inside a scrolling div", () => __awaiter(this, void 0, void 0, function* () {
                const imageName = "internal-scroll.png";
                const baseline = `./test/public/img/test.png`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#scroll"));
                yield compareImageDimensions(baseline, current);
            }));
            it("should take a screenshot of an element inside a scrolling div with padding", () => __awaiter(this, void 0, void 0, function* () {
                const imageName = "internal-scroll-padding.png";
                const baseline = `./test/public/img/test.png`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#scroll-padding"));
                yield compareImageDimensions(baseline, current);
            }));
            it("should take a screenshot of the content inside a scrolling div", () => __awaiter(this, void 0, void 0, function* () {
                const imageName = "internal-scroll-content.png";
                const baseline = `./test/public/img/test.png`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#scroll-content"), { elementContent: true });
                yield compareImageDimensions(baseline, current);
            }));
            it("should take a screenshot of an absolutely positioned element", () => __awaiter(this, void 0, void 0, function* () {
                const imageName = "internal-scroll-absolute.png";
                const baseline = `./test/public/img/test.png`;
                const current = `${suitePath}/${imageName}`;
                yield snappit_1.snap(imageName, snappit_1.$("#scroll-absolute"));
                yield compareImageDimensions(baseline, current);
            }));
        });
        describe("blacking out elements", () => {
            let previousHtml;
            let previousHead;
            const getOuterHtml = "return arguments[0].outerHTML";
            before(() => __awaiter(this, void 0, void 0, function* () {
                config.logException = [errors_1.ScreenshotExceptionName.NO_BASELINE];
                snappit = new snappit_1.Snappit(config);
                driver = yield snappit.start();
                yield driver.get("http://localhost:8080/blackout-elements");
                previousHtml = (yield driver.executeScript(getOuterHtml, snappit_1.$("#blackout")));
                previousHead = (yield driver.executeScript(getOuterHtml, snappit_1.$("head")));
            }));
            after(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
            it("should black out one element", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit_1.snap("blackout-one", snappit_1.$("#blackout"), { hide: [snappit_1.$("#hide0")] });
            }));
            it("should reset the styles and the html when removing blackout styles", () => __awaiter(this, void 0, void 0, function* () {
                const currentHtml = yield driver.executeScript(getOuterHtml, snappit_1.$("#blackout"));
                const currentHead = yield driver.executeScript(getOuterHtml, snappit_1.$("head"));
                chai_1.expect(currentHtml).to.eql(previousHtml);
                chai_1.expect(currentHead).to.eql(previousHead);
            }));
            it("should black out multiple elements", () => __awaiter(this, void 0, void 0, function* () {
                yield snappit_1.snap("blackout-many", snappit_1.$("#blackout"), {
                    hide: yield driver.findElements(selenium_webdriver_1.By.css("img[id^='hide']")),
                });
            }));
            it("should reset the styles and the html when removing blackout styles on multiple elements", () => __awaiter(this, void 0, void 0, function* () {
                const currentHtml = yield driver.executeScript(getOuterHtml, snappit_1.$("#blackout"));
                const currentHead = yield driver.executeScript(getOuterHtml, snappit_1.$("head"));
                chai_1.expect(currentHtml).to.eql(previousHtml);
                chai_1.expect(currentHead).to.eql(previousHead);
            }));
        });
        describe("re-configuration", () => {
            before(() => __awaiter(this, void 0, void 0, function* () {
                driver = yield snappit.start();
                yield driver.get("http://localhost:8080/");
            }));
            after(() => __awaiter(this, void 0, void 0, function* () {
                yield snappit.stop();
            }));
            it("should reject invalid thresholds", () => {
                const fn = () => {
                    snappit_1.snap.configure({ threshold: 1.01 });
                };
                chai_1.expect(fn).to.throw('Configuration error: Please set a "threshold" between 0 and 0.99');
            });
            it("should not trigger a baseline threshold error when setting the threshold very high", () => __awaiter(this, void 0, void 0, function* () {
                snappit_1.snap.configure({ threshold: 0.99 });
                snappit_1.$("#toggle-button").click();
                const error = yield snappit_1.snap("different-below-threshold.png", snappit_1.$("#color-div")).catch((err) => err);
                chai_1.expect(error).to.equal(undefined);
            }));
            it("should trigger a baseline threshold error when setting the threshold very low", () => __awaiter(this, void 0, void 0, function* () {
                snappit_1.snap.configure({ threshold: 0.001 });
                snappit_1.$("#border-button").click();
                const error = yield snappit_1.snap("different-below-threshold.png", snappit_1.$("#color-div")).catch((err) => err);
                chai_1.expect(error).to.be.instanceOf(errors_1.ScreenshotMismatchException);
            }));
            it("should not throw an error but still save if the screenshot does not exist", () => __awaiter(this, void 0, void 0, function* () {
                snappit_1.snap.configure({ logException: [errors_1.ScreenshotExceptionName.NO_BASELINE] });
                yield snappit_1.snap("throw-no-baseline-false.png", snappit_1.$("#color-div"));
            }));
            it("should not throw an error but still save if the screenshot is a different size", () => __awaiter(this, void 0, void 0, function* () {
                snappit_1.snap.configure({ logException: [errors_1.ScreenshotExceptionName.SIZE_DIFFERENCE] });
                yield snappit_1.snap("throw-size-difference-false.png").catch((err) => err);
                yield snappit_1.snap("throw-size-difference-false.png", snappit_1.$("#color-div"));
            }));
            it("should not throw an error but still save if the screenshot does not match", () => __awaiter(this, void 0, void 0, function* () {
                snappit_1.snap.configure({ logException: [errors_1.ScreenshotExceptionName.MISMATCH] });
                yield snappit_1.snap("throw-no-mismatch-false.png", snappit_1.$("#color-div")).catch((err) => err);
                snappit_1.$("#border-button").click();
                yield snappit_1.snap("throw-no-mismatch-false.png", snappit_1.$("#color-div"));
            }));
            it("should take a screenshot with directory and path tokens", () => __awaiter(this, void 0, void 0, function* () {
                const size = yield driver.manage().window().getSize();
                const capabilities = yield driver.getCapabilities();
                const version = (capabilities.get("version") || capabilities.get("browserVersion"))
                    .replace(/\W+/gi, "-");
                const tokenPath = `${config.browser}/${version}/${size.width}x${size.height}/test.png`;
                // This will error because the screenshot does not already exist, but we only care if it's created.
                yield snappit_1.snap("{browserName}/{browserVersion}/{browserSize}/test.png").catch((err) => err);
                yield snappit_1.snap(tokenPath);
            }));
        });
    });
}
// tslint:disable-next-line:no-namespace
(function (browserTest) {
    function only(suiteName, config) {
        return browserTest(suiteName, config, describe.only);
    }
    browserTest.only = only;
    function skip(suiteName, config) {
        return browserTest(suiteName, config, describe.skip);
    }
    browserTest.skip = skip;
})(browserTest || (browserTest = {}));
describe("Snappit", () => {
    browserTest("Chrome", {
        browser: "chrome",
    });
    browserTest("GeckoDriver FireFox", {
        browser: "firefox",
    });
    browserTest("Chrome Headless", {
        browser: "chrome",
        headless: true,
    });
    browserTest("GeckoDriver FireFox Headless", {
        browser: "firefox",
        headless: true,
    });
});
//# sourceMappingURL=snappit.spec.js.map